@@ -26,9 +26,6 @@
 # OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 # OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
-# no unicode literals
-from __future__ import absolute_import, division, print_function
-
 
 def parse_version(vstr):
     res = 0
@@ -55,8 +52,8 @@ def check(version, name):
 
 
 def synthesize(vers, opts):
-    """ Synthesize a capability enabled version response
-        This is a very limited emulation for relatively recent feature sets
+    """Synthesize a capability enabled version response
+    This is a very limited emulation for relatively recent feature sets
     """
     parsed_version = parse_version(vers["version"])
     vers["capabilities"] = {}
