@@ -5,7 +5,6 @@
 
 '''commands to sign and verify changesets'''
 
-from __future__ import absolute_import
 
 import binascii
 import os
@@ -14,7 +13,6 @@ from mercurial.i18n import _
 from mercurial.node import (
     bin,
     hex,
-    nullid,
     short,
 )
 from mercurial import (
@@ -66,7 +64,7 @@ help.CATEGORY_ORDER.insert(
 help.CATEGORY_NAMES[_HELP_CATEGORY] = b'Signing changes (GPG)'
 
 
-class gpg(object):
+class gpg:
     def __init__(self, path, key=None):
         self.path = path
         self.key = (key and b" --local-user \"%s\"" % key) or b""
@@ -76,7 +74,7 @@ class gpg(object):
         return procutil.filter(data, gpgcmd)
 
     def verify(self, data, sig):
-        """ returns of the good and bad signatures"""
+        """returns of the good and bad signatures"""
         sigfile = datafile = None
         try:
             # create temporary files
@@ -314,7 +312,9 @@ def _dosign(ui, repo, *revs, **opts):
     if revs:
         nodes = [repo.lookup(n) for n in revs]
     else:
-        nodes = [node for node in repo.dirstate.parents() if node != nullid]
+        nodes = [
+            node for node in repo.dirstate.parents() if node != repo.nullid
+        ]
         if len(nodes) > 1:
             raise error.Abort(
                 _(b'uncommitted merge - please provide a specific revision')
@@ -352,7 +352,8 @@ def _dosign(ui, repo, *revs, **opts):
     sigsfile.close()
 
     if b'.hgsigs' not in repo.dirstate:
-        repo[None].add([b".hgsigs"])
+        with repo.dirstate.changing_files(repo):
+            repo[None].add([b".hgsigs"])
 
     if opts[b"no_commit"]:
         return
