@@ -5,22 +5,17 @@
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2 or any later version.
 
-from __future__ import absolute_import
 
 import os
 
 from .i18n import _
-from .node import (
-    nullid,
-    short,
-)
-from .utils import (
-    stringutil,
-)
+from .node import short
+from .utils import stringutil
 
 from . import (
     error,
     pycompat,
+    requirements,
     revlog,
     util,
 )
@@ -43,7 +38,24 @@ def _normpath(f):
     return f
 
 
-class verifier(object):
+HINT_FNCACHE = _(
+    b'hint: run "hg debugrebuildfncache" to recover from corrupt fncache\n'
+)
+
+WARN_PARENT_DIR_UNKNOWN_REV = _(
+    b"parent-directory manifest refers to unknown revision %s"
+)
+
+WARN_UNKNOWN_COPY_SOURCE = _(
+    b"warning: copy source of '%s' not in parents of %s"
+)
+
+WARN_NULLID_COPY_SOURCE = _(
+    b"warning: %s@%s: copy source revision is nullid %s:%s\n"
+)
+
+
+class verifier:
     def __init__(self, repo, level=None):
         self.repo = repo.unfiltered()
         self.ui = repo.ui
@@ -56,7 +68,7 @@ class verifier(object):
         self.warnings = 0
         self.havecl = len(repo.changelog) > 0
         self.havemf = len(repo.manifestlog.getstorage(b'')) > 0
-        self.revlogv1 = repo.changelog.version != revlog.REVLOGV0
+        self.revlogv1 = repo.changelog._format_version != revlog.REVLOGV0
         self.lrugetctx = util.lrucachefunc(repo.unfiltered().__getitem__)
         self.refersmf = False
         self.fncachewarned = False
@@ -107,7 +119,7 @@ class verifier(object):
         if d[1]:
             self._err(None, _(b"index contains %d extra bytes") % d[1], name)
 
-        if obj.version != revlog.REVLOGV0:
+        if obj._format_version != revlog.REVLOGV0:
             if not self.revlogv1:
                 self._warn(_(b"warning: `%s' uses revlog format 1") % name)
         elif self.revlogv1:
@@ -119,7 +131,7 @@ class verifier(object):
         arguments are:
         - obj:      the source revlog
         - i:        the revision number
-        - node:        the revision node id
+        - node:     the revision node id
         - seen:     nodes previously seen for this revlog
         - linkrevs: [changelog-revisions] introducing "node"
         - f:        string label ("changelog", "manifest", or filename)
@@ -144,33 +156,25 @@ class verifier(object):
                 if f and len(linkrevs) > 1:
                     try:
                         # attempt to filter down to real linkrevs
-                        linkrevs = [
-                            l
-                            for l in linkrevs
-                            if self.lrugetctx(l)[f].filenode() == node
-                        ]
+                        linkrevs = []
+                        for lr in linkrevs:
+                            if self.lrugetctx(lr)[f].filenode() == node:
+                                linkrevs.append(lr)
                     except Exception:
                         pass
-                self._warn(
-                    _(b" (expected %s)")
-                    % b" ".join(map(pycompat.bytestr, linkrevs))
-                )
+                msg = _(b" (expected %s)")
+                msg %= b" ".join(map(pycompat.bytestr, linkrevs))
+                self._warn(msg)
             lr = None  # can't be trusted
 
         try:
             p1, p2 = obj.parents(node)
-            if p1 not in seen and p1 != nullid:
-                self._err(
-                    lr,
-                    _(b"unknown parent 1 %s of %s") % (short(p1), short(node)),
-                    f,
-                )
-            if p2 not in seen and p2 != nullid:
-                self._err(
-                    lr,
-                    _(b"unknown parent 2 %s of %s") % (short(p2), short(node)),
-                    f,
-                )
+            if p1 not in seen and p1 != self.repo.nullid:
+                msg = _(b"unknown parent 1 %s of %s") % (short(p1), short(node))
+                self._err(lr, msg, f)
+            if p2 not in seen and p2 != self.repo.nullid:
+                msg = _(b"unknown parent 2 %s of %s") % (short(p2), short(node))
+                self._err(lr, msg, f)
         except Exception as inst:
             self._exc(lr, _(b"checking parents of %s") % short(node), inst, f)
 
@@ -207,6 +211,12 @@ class verifier(object):
         self._crosscheckfiles(filelinkrevs, filenodes)
         totalfiles, filerevisions = self._verifyfiles(filenodes, filelinkrevs)
 
+        if self.errors:
+            ui.warn(_(b"not checking dirstate because of previous errors\n"))
+            dirstate_errors = 0
+        else:
+            dirstate_errors = self._verify_dirstate()
+
         # final report
         ui.status(
             _(b"checked %d changesets with %d changes to %d files\n")
@@ -215,19 +225,18 @@ class verifier(object):
         if self.warnings:
             ui.warn(_(b"%d warnings encountered!\n") % self.warnings)
         if self.fncachewarned:
-            ui.warn(
-                _(
-                    b'hint: run "hg debugrebuildfncache" to recover from '
-                    b'corrupt fncache\n'
-                )
-            )
+            ui.warn(HINT_FNCACHE)
         if self.errors:
             ui.warn(_(b"%d integrity errors encountered!\n") % self.errors)
             if self.badrevs:
+                msg = _(b"(first damaged changeset appears to be %d)\n")
+                msg %= min(self.badrevs)
+                ui.warn(msg)
+            if dirstate_errors:
                 ui.warn(
-                    _(b"(first damaged changeset appears to be %d)\n")
-                    % min(self.badrevs)
+                    _(b"dirstate inconsistent with current parent's manifest\n")
                 )
+                ui.warn(_(b"%d dirstate errors\n") % dirstate_errors)
             return 1
         return 0
 
@@ -267,7 +276,7 @@ class verifier(object):
 
             try:
                 changes = cl.read(n)
-                if changes[0] != nullid:
+                if changes[0] != self.repo.nullid:
                     mflinkrevs.setdefault(changes[0], []).append(i)
                     self.refersmf = True
                 for f in changes[3]:
@@ -331,7 +340,7 @@ class verifier(object):
         if self.refersmf:
             # Do not check manifest if there are only changelog entries with
             # null manifests.
-            self._checkrevlog(mf, label, 0)
+            self._checkrevlog(mf._revlog, label, 0)
         progress = ui.makeprogress(
             _(b'checking'), unit=_(b'manifests'), total=len(mf)
         )
@@ -343,11 +352,8 @@ class verifier(object):
             if n in mflinkrevs:
                 del mflinkrevs[n]
             elif dir:
-                self._err(
-                    lr,
-                    _(b"%s not in parent-directory manifest") % short(n),
-                    label,
-                )
+                msg = _(b"%s not in parent-directory manifest") % short(n)
+                self._err(lr, msg, label)
             else:
                 self._err(lr, _(b"%s not in changesets") % short(n), label)
 
@@ -362,9 +368,8 @@ class verifier(object):
                     if fl == b't':
                         if not match.visitdir(fullpath):
                             continue
-                        subdirnodes.setdefault(fullpath + b'/', {}).setdefault(
-                            fn, []
-                        ).append(lr)
+                        sdn = subdirnodes.setdefault(fullpath + b'/', {})
+                        sdn.setdefault(fn, []).append(lr)
                     else:
                         if not match(fullpath):
                             continue
@@ -378,12 +383,8 @@ class verifier(object):
                     # code (eg: hash verification, filename are ordered, etc.)
                     mfdelta = mfl.get(dir, n).read()
                 except Exception as inst:
-                    self._exc(
-                        lr,
-                        _(b"reading full manifest %s") % short(n),
-                        inst,
-                        label,
-                    )
+                    msg = _(b"reading full manifest %s") % short(n)
+                    self._exc(lr, msg, inst, label)
 
         if not dir:
             progress.complete()
@@ -394,43 +395,33 @@ class verifier(object):
             changesetpairs = [(c, m) for m in mflinkrevs for c in mflinkrevs[m]]
             for c, m in sorted(changesetpairs):
                 if dir:
-                    self._err(
-                        c,
-                        _(
-                            b"parent-directory manifest refers to unknown"
-                            b" revision %s"
-                        )
-                        % short(m),
-                        label,
-                    )
+                    self._err(c, WARN_PARENT_DIR_UNKNOWN_REV % short(m), label)
                 else:
-                    self._err(
-                        c,
-                        _(b"changeset refers to unknown revision %s")
-                        % short(m),
-                        label,
-                    )
+                    msg = _(b"changeset refers to unknown revision %s")
+                    msg %= short(m)
+                    self._err(c, msg, label)
 
         if not dir and subdirnodes:
             self.ui.status(_(b"checking directory manifests\n"))
             storefiles = set()
             subdirs = set()
             revlogv1 = self.revlogv1
-            for t, f, f2, size in repo.store.datafiles():
-                if not f:
-                    self._err(None, _(b"cannot decode filename '%s'") % f2)
-                elif (size > 0 or not revlogv1) and f.startswith(b'meta/'):
+            undecodable = []
+            for t, f, size in repo.store.datafiles(undecodable=undecodable):
+                if (size > 0 or not revlogv1) and f.startswith(b'meta/'):
                     storefiles.add(_normpath(f))
                     subdirs.add(os.path.dirname(f))
+            for f in undecodable:
+                self._err(None, _(b"cannot decode filename '%s'") % f)
             subdirprogress = ui.makeprogress(
                 _(b'checking'), unit=_(b'manifests'), total=len(subdirs)
             )
 
-        for subdir, linkrevs in pycompat.iteritems(subdirnodes):
+        for subdir, linkrevs in subdirnodes.items():
             subdirfilenodes = self._verifymanifest(
                 linkrevs, subdir, storefiles, subdirprogress
             )
-            for f, onefilenodes in pycompat.iteritems(subdirfilenodes):
+            for f, onefilenodes in subdirfilenodes.items():
                 filenodes.setdefault(f, {}).update(onefilenodes)
 
         if not dir and subdirnodes:
@@ -480,15 +471,16 @@ class verifier(object):
         ui.status(_(b"checking files\n"))
 
         storefiles = set()
-        for rl_type, f, f2, size in repo.store.datafiles():
-            if not f:
-                self._err(None, _(b"cannot decode filename '%s'") % f2)
-            elif (size > 0 or not revlogv1) and f.startswith(b'data/'):
+        undecodable = []
+        for t, f, size in repo.store.datafiles(undecodable=undecodable):
+            if (size > 0 or not revlogv1) and f.startswith(b'data/'):
                 storefiles.add(_normpath(f))
+        for f in undecodable:
+            self._err(None, _(b"cannot decode filename '%s'") % f)
 
         state = {
             # TODO this assumes revlog storage for changelog.
-            b'expectedversion': self.repo.changelog.version & 0xFFFF,
+            b'expectedversion': self.repo.changelog._format_version,
             b'skipflags': self.skipflags,
             # experimental config: censor.policy
             b'erroroncensored': ui.config(b'censor', b'policy') == b'abort',
@@ -523,9 +515,8 @@ class verifier(object):
                     storefiles.remove(ff)
                 except KeyError:
                     if self.warnorphanstorefiles:
-                        self._warn(
-                            _(b" warning: revlog '%s' not in fncache!") % ff
-                        )
+                        msg = _(b" warning: revlog '%s' not in fncache!")
+                        self._warn(msg % ff)
                         self.fncachewarned = True
 
             if not len(fl) and (self.havecl or self.havemf):
@@ -544,11 +535,8 @@ class verifier(object):
                     if problem.warning:
                         self._warn(problem.warning)
                     elif problem.error:
-                        self._err(
-                            linkrev if linkrev is not None else lr,
-                            problem.error,
-                            f,
-                        )
+                        linkrev_msg = linkrev if linkrev is not None else lr
+                        self._err(linkrev_msg, problem.error, f)
                     else:
                         raise error.ProgrammingError(
                             b'problem instance does not set warning or error '
@@ -580,32 +568,15 @@ class verifier(object):
                         if lr is not None and ui.verbose:
                             ctx = lrugetctx(lr)
                             if not any(rp[0] in pctx for pctx in ctx.parents()):
-                                self._warn(
-                                    _(
-                                        b"warning: copy source of '%s' not"
-                                        b" in parents of %s"
-                                    )
-                                    % (f, ctx)
-                                )
+                                self._warn(WARN_UNKNOWN_COPY_SOURCE % (f, ctx))
                         fl2 = repo.file(rp[0])
                         if not len(fl2):
-                            self._err(
-                                lr,
-                                _(
-                                    b"empty or missing copy source revlog "
-                                    b"%s:%s"
-                                )
-                                % (rp[0], short(rp[1])),
-                                f,
-                            )
-                        elif rp[1] == nullid:
-                            ui.note(
-                                _(
-                                    b"warning: %s@%s: copy source"
-                                    b" revision is nullid %s:%s\n"
-                                )
-                                % (f, lr, rp[0], short(rp[1]))
-                            )
+                            m = _(b"empty or missing copy source revlog %s:%s")
+                            self._err(lr, m % (rp[0], short(rp[1])), f)
+                        elif rp[1] == self.repo.nullid:
+                            msg = WARN_NULLID_COPY_SOURCE
+                            msg %= (f, lr, rp[0], short(rp[1]))
+                            ui.note(msg)
                         else:
                             fl2.rev(rp[1])
                 except Exception as inst:
@@ -615,14 +586,10 @@ class verifier(object):
 
             # cross-check
             if f in filenodes:
-                fns = [(v, k) for k, v in pycompat.iteritems(filenodes[f])]
+                fns = [(v, k) for k, v in filenodes[f].items()]
                 for lr, node in sorted(fns):
-                    self._err(
-                        lr,
-                        _(b"manifest refers to unknown revision %s")
-                        % short(node),
-                        f,
-                    )
+                    msg = _(b"manifest refers to unknown revision %s")
+                    self._err(lr, msg % short(node), f)
         progress.complete()
 
         if self.warnorphanstorefiles:
@@ -630,3 +597,25 @@ class verifier(object):
                 self._warn(_(b"warning: orphan data file '%s'") % f)
 
         return len(files), revisions
+
+    def _verify_dirstate(self):
+        """Check that the dirstate is consistent with the parent's manifest"""
+        repo = self.repo
+        ui = self.ui
+        ui.status(_(b"checking dirstate\n"))
+
+        parent1, parent2 = repo.dirstate.parents()
+        m1 = repo[parent1].manifest()
+        m2 = repo[parent2].manifest()
+        dirstate_errors = 0
+
+        is_narrow = requirements.NARROW_REQUIREMENT in repo.requirements
+        narrow_matcher = repo.narrowmatch() if is_narrow else None
+
+        for err in repo.dirstate.verify(m1, m2, parent1, narrow_matcher):
+            ui.error(err)
+            dirstate_errors += 1
+
+        if dirstate_errors:
+            self.errors += dirstate_errors
+        return dirstate_errors
